@@ -1,18 +1,17 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
 
-"""
+'''
 PyTextile
 
 A Humane Web Text Generator
-"""
+'''
 
 # Last upstream version basis
 # __version__ = '2.1.4'
 # __date__ = '2009/12/04'
 
-__copyright__ = """
+__copyright__ = '''
 Copyright (c) 2011, Leigh Parry <leighparry@blueyonder.co.uk>
 Copyright (c) 2011, John Schember <john@nachtimwald.com>
 Copyright (c) 2009, Jason Samsa, http://jsamsa.com/
@@ -28,9 +27,9 @@ Textile's procedural code into a class framework
 
 Additions and fixes Copyright (c) 2006 Alex Shiels http://thresholdstate.com/
 
-"""
+'''
 
-__license__ = """
+__license__ = '''
 L I C E N S E
 =============
 Redistribution and use in source and binary forms, with or without
@@ -59,7 +58,7 @@ CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
-"""
+'''
 
 import re
 import uuid
@@ -77,7 +76,7 @@ def _normalize_newlines(string):
 
 
 def getimagesize(url):
-    """
+    '''
     Attempts to determine an image's width and height, and returns a string
     suitable for use in an <img> tag, or None in case of failure.
     Requires that PIL is installed.
@@ -86,7 +85,7 @@ def getimagesize(url):
     ... #doctest: +ELLIPSIS, +SKIP
     'width="..." height="..."'
 
-    """
+    '''
 
     from PIL import ImageFile
 
@@ -99,7 +98,7 @@ def getimagesize(url):
                 break
             p.feed(s)
             if p.image:
-                return 'width="%i" height="%i"' % p.image.size
+                return f'width="{p.image.size[0]}" height="{p.image.size[1]}"'
     except (OSError, ValueError):
         return None
 
@@ -112,9 +111,9 @@ class Textile:
     styl = r'(?:\{[^}]+\})'
     cspn = r'(?:\\\d+)'
     rspn = r'(?:\/\d+)'
-    a = r'(?:%s|%s)*' % (hlgn, vlgn)
-    s = r'(?:%s|%s)*' % (cspn, rspn)
-    c = r'(?:%s)*' % '|'.join([clas, styl, lnge, hlgn])
+    a = fr'(?:{hlgn}|{vlgn})*'
+    s = fr'(?:{cspn}|{rspn})*'
+    c = r'(?:{})*'.format('|'.join([clas, styl, lnge, hlgn]))
 
     pnct = r'[-!"#$%&()*+,/:;<=>?@\'\[\\\]\.^_`{|}~]'
     # urlch = r'[\w"$\-_.+!*\'(),";/?:@=&%#{}|\\^~\[\]`]'
@@ -263,7 +262,7 @@ class Textile:
     ]
 
     def __init__(self, restricted=False, lite=False, noimage=False):
-        """docstring for __init__"""
+        '''docstring for __init__'''
         self.restricted = restricted
         self.lite = lite
         self.noimage = noimage
@@ -275,21 +274,21 @@ class Textile:
         self.html_type = 'xhtml'
 
     def textile(self, text, rel=None, head_offset=0, html_type='xhtml'):
-        """
+        '''
         >>> import textile
         >>> textile.textile('some textile')
-        u'\\t<p>some textile</p>'
-        """
+        '\\t<p>some textile</p>'
+        '''
         self.html_type = html_type
 
-        # text = type(u'')(text)
+        # text = str(text)
         text = _normalize_newlines(text)
 
         if self.restricted:
             text = self.encode_html(text, quotes=False)
 
         if rel:
-            self.rel = ' rel="%s"' % rel
+            self.rel = f' rel="{rel}"'
 
         text = self.getRefs(text)
         text = self.block(text, int(head_offset))
@@ -299,7 +298,7 @@ class Textile:
         return text
 
     def pba(self, input, element=None):
-        """
+        '''
         Parse block attributes.
 
         >>> t = Textile()
@@ -339,7 +338,7 @@ class Textile:
         >>> t.pba('[fr]')
         ' lang="fr"'
 
-        """
+        '''
         style = []
         aclass = ''
         lang = ''
@@ -361,9 +360,9 @@ class Textile:
                 rowspan = m.group(1)
 
         if element == 'td' or element == 'tr':
-            m = re.search(r'(%s)' % self.vlgn, matched)
+            m = re.search(rf'({self.vlgn})', matched)
             if m:
-                style.append("vertical-align:%s;" % self.vAlign(m.group(1)))
+                style.append(f'vertical-align:{self.vAlign(m.group(1))};')
 
         m = re.search(r'\{([^}]*)\}', matched)
         if m:
@@ -382,17 +381,17 @@ class Textile:
 
         m = re.search(r'([(]+)', matched)
         if m:
-            style.append("padding-left:%sem;" % len(m.group(1)))
+            style.append(f'padding-left:{len(m.group(1))}em;')
             matched = matched.replace(m.group(0), '')
 
         m = re.search(r'([)]+)', matched)
         if m:
-            style.append("padding-right:%sem;" % len(m.group(1)))
+            style.append(f'padding-right:{len(m.group(1))}em;')
             matched = matched.replace(m.group(0), '')
 
-        m = re.search(r'(%s)' % self.hlgn, matched)
+        m = re.search(rf'({self.hlgn})', matched)
         if m:
-            style.append("text-align:%s;" % self.hAlign(m.group(1)))
+            style.append(f'text-align:{self.hAlign(m.group(1))};')
 
         m = re.search(r'^(.*)#(.*)$', aclass)
         if m:
@@ -407,21 +406,21 @@ class Textile:
 
         result = []
         if style:
-            result.append(' style="%s"' % "".join(style))
+            result.append(' style="{}"'.format(''.join(style)))
         if aclass:
-            result.append(' class="%s"' % aclass)
+            result.append(f' class="{aclass}"')
         if lang:
-            result.append(' lang="%s"' % lang)
+            result.append(f' lang="{lang}"')
         if id:
-            result.append(' id="%s"' % id)
+            result.append(f' id="{id}"')
         if colspan:
-            result.append(' colspan="%s"' % colspan)
+            result.append(f' colspan="{colspan}"')
         if rowspan:
-            result.append(' rowspan="%s"' % rowspan)
+            result.append(f' rowspan="{rowspan}"')
         return ''.join(result)
 
     def hasRawText(self, text):
-        """
+        '''
         checks whether the text has text not already enclosed by a block tag
 
         >>> t = Textile()
@@ -431,26 +430,26 @@ class Textile:
         >>> t.hasRawText(' why yes, yes it does')
         True
 
-        """
+        '''
         r = re.compile(r'<(p|blockquote|div|form|table|ul|ol|pre|h\d)[^>]*?>.*</\1>', re.S).sub('', text.strip()).strip()
         r = re.compile(r'<(hr|br)[^>]*?/>').sub('', r)
         return '' != r
 
     def table(self, text):
-        r"""
+        r'''
         >>> t = Textile()
         >>> t.table('|one|two|three|\n|a|b|c|')
         '\t<table>\n\t\t<tr>\n\t\t\t<td>one</td>\n\t\t\t<td>two</td>\n\t\t\t<td>three</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td>a</td>\n\t\t\t<td>b</td>\n\t\t\t<td>c</td>\n\t\t</tr>\n\t</table>\n\n'
-        """
-        text = text + "\n\n"
-        pattern = re.compile(r'^(?:table(_?%(s)s%(a)s%(c)s)\. ?\n)?^(%(a)s%(c)s\.? ?\|.*\|)\n\n' % {'s':self.s, 'a':self.a, 'c':self.c}, re.S|re.M|re.U)
+        '''
+        text = text + '\n\n'
+        pattern = re.compile(rf'^(?:table(_?{self.s}{self.a}{self.c})\. ?\n)?^({self.a}{self.c}\.? ?\|.*\|)\n\n', re.S|re.M|re.U)
         return pattern.sub(self.fTable, text)
 
     def fTable(self, match):
         tatts = self.pba(match.group(1), 'table')
         rows = []
         for row in [x for x in match.group(2).split('\n') if x]:
-            rmtch = re.search(r'^(%s%s\. )(.*)' % (self.a, self.c), row.lstrip())
+            rmtch = re.search(fr'^({self.a}{self.c}\. )(.*)', row.lstrip())
             if rmtch:
                 ratts = self.pba(rmtch.group(1), 'tr')
                 row = rmtch.group(2)
@@ -461,8 +460,8 @@ class Textile:
             for cell in row.split('|')[1:-1]:
                 ctyp = 'd'
                 if re.search(r'^_', cell):
-                    ctyp = "h"
-                cmtch = re.search(r'^(_?%s%s%s\. )(.*)' % (self.s, self.a, self.c), cell)
+                    ctyp = 'h'
+                cmtch = re.search(fr'^(_?{self.s}{self.a}{self.c}\. )(.*)', cell)
                 if cmtch:
                     catts = self.pba(cmtch.group(1), 'td')
                     cell = cmtch.group(2)
@@ -470,23 +469,23 @@ class Textile:
                     catts = ''
 
                 cell = self.graf(self.span(cell))
-                cells.append('\t\t\t<t%s%s>%s</t%s>' % (ctyp, catts, cell, ctyp))
-            rows.append("\t\t<tr%s>\n%s\n\t\t</tr>" % (ratts, '\n'.join(cells)))
+                cells.append(f'\t\t\t<t{ctyp}{catts}>{cell}</t{ctyp}>')
+            rows.append('\t\t<tr{}>\n{}\n\t\t</tr>'.format(ratts, '\n'.join(cells)))
             cells = []
             catts = None
-        return "\t<table%s>\n%s\n\t</table>\n\n" % (tatts, '\n'.join(rows))
+        return '\t<table{}>\n{}\n\t</table>\n\n'.format(tatts, '\n'.join(rows))
 
     def lists(self, text):
-        """
+        '''
         >>> t = Textile()
         >>> t.lists("* one\\n* two\\n* three")
         '\\t<ul>\\n\\t\\t<li>one</li>\\n\\t\\t<li>two</li>\\n\\t\\t<li>three</li>\\n\\t</ul>'
-        """
-        pattern = re.compile(r'^([#*]+%s .*)$(?![^#*])' % self.c, re.U|re.M|re.S)
+        '''
+        pattern = re.compile(rf'^([#*]+{self.c} .*)$(?![^#*])', re.U|re.M|re.S)
         return pattern.sub(self.fList, text)
 
     def fList(self, match):
-        text = match.group(0).split("\n")
+        text = match.group(0).split('\n')
         result = []
         lists = []
         for i, line in enumerate(text):
@@ -495,7 +494,7 @@ class Textile:
             except IndexError:
                 nextline = ''
 
-            m = re.search(r"^([#*]+)(%s%s) (.*)$" % (self.a, self.c), line, re.S)
+            m = re.search(fr'^([#*]+)({self.a}{self.c}) (.*)$', line, re.S)
             if m:
                 tl, atts, content = m.groups()
                 nl = ''
@@ -505,21 +504,21 @@ class Textile:
                 if tl not in lists:
                     lists.append(tl)
                     atts = self.pba(atts)
-                    line = "\t<%sl%s>\n\t\t<li>%s" % (self.lT(tl), atts, self.graf(content))
+                    line = f'\t<{self.lT(tl)}l{atts}>\n\t\t<li>{self.graf(content)}'
                 else:
-                    line = "\t\t<li>" + self.graf(content)
+                    line = '\t\t<li>' + self.graf(content)
 
                 if len(nl) <= len(tl):
-                    line = line + "</li>"
+                    line = line + '</li>'
                 for k in reversed(lists):
                     if len(k) > len(nl):
-                        line = line + "\n\t</%sl>" % self.lT(k)
+                        line = line + f'\n\t</{self.lT(k)}l>'
                         if len(k) > 1:
-                            line = line + "</li>"
+                            line = line + '</li>'
                         lists.remove(k)
 
             result.append(line)
-        return "\n".join(result)
+        return '\n'.join(result)
 
     def lT(self, input):
         if re.search(r'^#+', input):
@@ -535,14 +534,14 @@ class Textile:
             content = re.sub(r'(.+)(?:(?<!<br>)|(?<!<br />))\n(?![#*\s|])', '\\1<br>', match.group(3))
         else:
             content = re.sub(r'(.+)(?:(?<!<br>)|(?<!<br />))\n(?![#*\s|])', '\\1<br />', match.group(3))
-        return '<%s%s>%s%s' % (match.group(1), match.group(2), content, match.group(4))
+        return f'<{match.group(1)}{match.group(2)}>{content}{match.group(4)}'
 
     def block(self, text, head_offset=0):
-        """
+        '''
         >>> t = Textile()
         >>> t.block('h1. foobar baby')
         '\\t<h1>foobar baby</h1>'
-        """
+        '''
         if not self.lite:
             tre = '|'.join(self.btag)
         else:
@@ -556,7 +555,7 @@ class Textile:
 
         anon = False
         for line in text:
-            pattern = r'^(%s)(%s%s)\.(\.?)(?::(\S+))? (.*)$' % (tre, self.a, self.c)
+            pattern = fr'^({tre})({self.a}{self.c})\.(\.?)(?::(\S+))? (.*)$'
             match = re.search(pattern, line, re.S)
             if match:
                 if ext:
@@ -566,18 +565,16 @@ class Textile:
                 h_match = re.search(r'h([1-6])', tag)
                 if h_match:
                     head_level, = h_match.groups()
-                    tag = 'h%i' % max(1,
-                                      min(int(head_level) + head_offset,
-                                          6))
+                    tag = f'h{max(1, min(int(head_level) + head_offset, 6))}'
                 o1, o2, content, c2, c1 = self.fBlock(tag, atts, ext,
                                                       cite, graf)
                 # leave off c1 if this block is extended,
                 # we'll close it at the start of the next block
 
                 if ext:
-                    line = "%s%s%s%s" % (o1, o2, content, c2)
+                    line = f'{o1}{o2}{content}{c2}'
                 else:
-                    line = "%s%s%s%s%s" % (o1, o2, content, c2, c1)
+                    line = f'{o1}{o2}{content}{c2}{c1}'
 
             else:
                 anon = True
@@ -589,16 +586,16 @@ class Textile:
                     if tag == 'p' and not self.hasRawText(content):
                         line = content
                     else:
-                        line = "%s%s%s" % (o2, content, c2)
+                        line = f'{o2}{content}{c2}'
                 else:
                     line = self.graf(line)
 
             line = self.doPBr(line)
             if self.html_type == 'xhtml':
-                line = re.sub(r'<br>', '<br />', line)
+                line = line.replace('<br>', '<br />')
 
             if ext and anon:
-                out.append(out.pop() + "\n" + line)
+                out.append(out.pop() + '\n' + line)
             else:
                 out.append(line)
 
@@ -613,7 +610,7 @@ class Textile:
         return '\n\n'.join(out)
 
     def fBlock(self, tag, atts, ext, cite, content):
-        """
+        '''
         >>> t = Textile()
         >>> t.fBlock("bq", "", None, "", "Hello BlockQuote")
         ('\\t<blockquote>\\n', '\\t\\t<p>', 'Hello BlockQuote', '</p>', '\\n\\t</blockquote>')
@@ -626,7 +623,7 @@ class Textile:
 
         >>> t.fBlock("h1", "", None, "", "foobar")
         ('', '\\t<h1>', 'foobar', '</h1>', '')
-        """
+        '''
         atts = self.pba(atts)
         o1 = o2 = c2 = c1 = ''
 
@@ -637,28 +634,28 @@ class Textile:
                 fnid = self.fn[m.group(1)]
             else:
                 fnid = m.group(1)
-            atts = atts + ' id="fn%s"' % fnid
+            atts = atts + f' id="fn{fnid}"'
             if atts.find('class=') < 0:
                 atts = atts + ' class="footnote"'
-            content = ('<sup>%s</sup>' % m.group(1)) + content
+            content = (f'<sup>{m.group(1)}</sup>') + content
 
         if tag == 'bq':
             cite = self.checkRefs(cite)
             if cite:
-                cite = ' cite="%s"' % cite
+                cite = f' cite="{cite}"'
             else:
                 cite = ''
-            o1 = "\t<blockquote%s%s>\n" % (cite, atts)
-            o2 = "\t\t<p%s>" % atts
-            c2 = "</p>"
-            c1 = "\n\t</blockquote>"
+            o1 = f'\t<blockquote{cite}{atts}>\n'
+            o2 = f'\t\t<p{atts}>'
+            c2 = '</p>'
+            c1 = '\n\t</blockquote>'
 
         elif tag == 'bc':
-            o1 = "<pre%s>" % atts
-            o2 = "<code%s>" % atts
-            c2 = "</code>"
-            c1 = "</pre>"
-            content = self.shelve(self.encode_html(content.rstrip("\n") + "\n"))
+            o1 = f'<pre{atts}>'
+            o2 = f'<code{atts}>'
+            c2 = '</code>'
+            c1 = '</pre>'
+            content = self.shelve(self.encode_html(content.rstrip('\n') + '\n'))
 
         elif tag == 'notextile':
             content = self.shelve(content)
@@ -666,24 +663,24 @@ class Textile:
             c1 = c2 = ''
 
         elif tag == 'pre':
-            content = self.shelve(self.encode_html(content.rstrip("\n") + "\n"))
-            o1 = "<pre%s>" % atts
+            content = self.shelve(self.encode_html(content.rstrip('\n') + '\n'))
+            o1 = f'<pre{atts}>'
             o2 = c2 = ''
             c1 = '</pre>'
 
         else:
-            o2 = "\t<%s%s>" % (tag, atts)
-            c2 = "</%s>" % tag
+            o2 = f'\t<{tag}{atts}>'
+            c2 = f'</{tag}>'
 
         content = self.graf(content)
         return o1, o2, content, c2, c1
 
     def footnoteRef(self, text):
-        """
+        '''
         >>> t = Textile()
         >>> t.footnoteRef('foo[1] ') # doctest: +ELLIPSIS
         'foo<sup class="footnote"><a href="#fn...">1</a></sup> '
-        """
+        '''
         return re.sub(r'\b\[([0-9]+)\](\s)?', self.footnoteID, text)
 
     def footnoteID(self, match):
@@ -693,10 +690,10 @@ class Textile:
         fnid = self.fn[id]
         if not t:
             t = ''
-        return '<sup class="footnote"><a href="#fn%s">%s</a></sup>%s' % (fnid, id, t)
+        return f'<sup class="footnote"><a href="#fn{fnid}">{id}</a></sup>{t}'
 
     def glyphs(self, text):
-        """
+        '''
         >>> t = Textile()
 
         >>> t.glyphs("apostrophe's")
@@ -717,9 +714,9 @@ class Textile:
         >>> t.glyphs("<p><cite>Cat's Cradle</cite> by Vonnegut</p>")
         '<p><cite>Cat&#8217;s Cradle</cite> by Vonnegut</p>'
 
-        """
+        '''
         # fix: hackish
-        text = re.sub(r'"\Z', '\" ', text)
+        text = re.sub(r'"\Z', '" ', text)
 
         result = []
         for line in re.compile(r'(<.*?>)', re.U).split(text):
@@ -736,7 +733,7 @@ class Textile:
 
     def macros_only(self, text):
         # fix: hackish
-        text = re.sub(r'"\Z', '\" ', text)
+        text = re.sub(r'"\Z', '" ', text)
 
         result = []
         for line in re.compile(r'(<.*?>)', re.U).split(text):
@@ -758,10 +755,10 @@ class Textile:
         return d.get(input, '')
 
     def getRefs(self, text):
-        """
+        '''
         what is this for?
-        """
-        pattern = re.compile(r'(?:(?<=^)|(?<=\s))\[(.+)\]((?:http(?:s?):\/\/|\/)\S+)(?=\s|$)', re.U)
+        '''
+        pattern = re.compile(r'(?:(?<=^)|(?<=\s))\[(.+)\]((?:http(?:s?)://|/)\S+)(?=\s|$)', re.U)
         text = pattern.sub(self.refs, text)
         return text
 
@@ -774,7 +771,7 @@ class Textile:
         return self.urlrefs.get(url, url)
 
     def isRelURL(self, url):
-        """
+        '''
         Identify relative urls.
 
         >>> t = Textile()
@@ -783,8 +780,8 @@ class Textile:
         >>> t.isRelURL("/foo")
         True
 
-        """
-        (scheme, netloc) = urlparse(url)[0:2]
+        '''
+        scheme, netloc = urlparse(url)[0:2]
         return not scheme and not netloc
 
     def relURL(self, url):
@@ -799,12 +796,12 @@ class Textile:
         return id
 
     def retrieve(self, text):
-        """
+        '''
         >>> t = Textile()
         >>> id = t.shelve("foobar")
         >>> t.retrieve(id)
         'foobar'
-        """
+        '''
         while True:
             old = text
             for k, v in self.shelf.items():
@@ -851,19 +848,19 @@ class Textile:
         return text.rstrip('\n')
 
     def links(self, text):
-        """
+        '''
         >>> t = Textile()
         >>> t.links('fooobar "Google":http://google.com/foobar/ and hello world "flickr":http://flickr.com/photos/jsamsa/ ') # doctest: +ELLIPSIS
         'fooobar ... and hello world ...'
-        """
+        '''
 
         text = self.macros_only(text)
         punct = '!"#$%&\'*+,-./:;=?@\\^_`|~'
 
-        pattern = r'''
-            (?P<pre>    [\s\[{(]|[%s]   )?
+        pattern = rf'''
+            (?P<pre>    [\s\[{{(]|[{re.escape(punct)}]   )?
             "                          # start
-            (?P<atts>   %s       )
+            (?P<atts>   {self.c}       )
             (?P<text>   [^"]+?   )
             \s?
             (?:   \(([^)]+?)\)(?=")   )?     # $title
@@ -871,7 +868,7 @@ class Textile:
             (?P<url>    (?:ftp|https?)? (?: :// )? [-A-Za-z0-9+&@#/?=~_()|!:,.;]*[-A-Za-z0-9+&@#/=~_()|]   )
             (?P<post>   [^\w\/;]*?   )
             (?=<|\s|$)
-        ''' % (re.escape(punct), self.c)
+        '''
 
         text = re.compile(pattern, re.X).sub(self.fLink, text)
 
@@ -893,7 +890,7 @@ class Textile:
 
         atts = self.pba(atts)
         if title:
-            atts = atts +  ' title="%s"' % self.encode_html(title)
+            atts = atts + f' title="{self.encode_html(title)}"'
 
         if not self.noimage:
             text = self.image(text)
@@ -902,31 +899,30 @@ class Textile:
         text = self.glyphs(text)
 
         url = self.relURL(url)
-        out = '<a href="%s"%s%s>%s</a>' % (self.encode_html(url), atts, self.rel, text)
+        out = f'<a href="{self.encode_html(url)}"{atts}{self.rel}>{text}</a>'
         out = self.shelve(out)
         return ''.join([pre, out, post])
 
     def span(self, text):
-        """
+        '''
         >>> t = Textile()
         >>> t.span(r"hello %(bob)span *strong* and **bold**% goodbye")
         'hello <span class="bob">span <strong>strong</strong> and <b>bold</b></span> goodbye'
-        """
+        '''
         qtags = (r'\*\*', r'\*', r'\?\?', r'\-', r'__', r'_', r'%', r'\+', r'~', r'\^')
         pnct = ".,\"'?!;:"
 
         for qtag in qtags:
-            pattern = re.compile(r"""
-                (?:^|(?<=[\s>%(pnct)s\(])|\[|([\]}]))
-                (%(qtag)s)(?!%(qtag)s)
-                (%(c)s)
+            pattern = re.compile(rf'''
+                (?:^|(?<=[\s>{pnct}\(])|\[|([\]}}]))
+                ({qtag})(?!{qtag})
+                ({self.c})
                 (?::(\S+))?
-                ([^\s%(qtag)s]+|\S[^%(qtag)s\n]*[^\s%(qtag)s\n])
-                ([%(pnct)s]*)
-                %(qtag)s
-                (?:$|([\]}])|(?=%(selfpnct)s{1,2}|\s))
-            """ % {'qtag':qtag, 'c':self.c, 'pnct':pnct,
-                   'selfpnct':self.pnct}, re.X)
+                ([^\s{qtag}]+|\S[^{qtag}\n]*[^\s{qtag}\n])
+                ([{pnct}]*)
+                {qtag}
+                (?:$|([\]}}])|(?={self.pnct}{{1,2}}|\s))
+            ''', re.X)
             text = pattern.sub(self.fSpan, text)
         return text
 
@@ -948,31 +944,31 @@ class Textile:
         tag = qtags[tag]
         atts = self.pba(atts)
         if cite:
-            atts = atts + 'cite="%s"' % cite
+            atts = atts + f'cite="{cite}"'
 
         content = self.span(content)
 
-        out = "<%s%s>%s%s</%s>" % (tag, atts, content, end, tag)
+        out = f'<{tag}{atts}>{content}{end}</{tag}>'
         return out
 
     def image(self, text):
-        """
+        '''
         >>> t = Textile()
         >>> t.image('!/imgs/myphoto.jpg!:http://jsamsa.com')
         '<a href="http://jsamsa.com"><img src="/imgs/myphoto.jpg" alt="" /></a>'
-        """
-        pattern = re.compile(r"""
-            (?:[\[{])?          # pre
+        '''
+        pattern = re.compile(rf'''
+            (?:[\[{{])?          # pre
             \!                 # opening !
-            (%s)               # optional style,class atts
+            ({self.c})               # optional style,class atts
             (?:\. )?           # optional dot-space
             ([^\s(!]+)         # presume this is the src
             \s?                # optional space
             (?:\(([^\)]+)\))?  # optional title
             \!                 # closing
             (?::(\S+))?        # optional href
-            (?:[\]}]|(?=\s|$)) # lookahead: space or end of string
-        """ % self.c, re.U|re.X)
+            (?:[\]}}]|(?=\s|$)) # lookahead: space or end of string
+        ''', re.U|re.X)
         return pattern.sub(self.fImage, text)
 
     def fImage(self, match):
@@ -981,14 +977,14 @@ class Textile:
         atts  = self.pba(atts)
 
         if title:
-            atts = atts + ' title="%s" alt="%s"' % (title, title)
+            atts = atts + f' title="{title}" alt="{title}"'
         else:
             atts = atts + ' alt=""'
 
         if not self.isRelURL(url) and self.get_sizes:
             size = getimagesize(url)
             if (size):
-                atts += " %s" % size
+                atts += f' {size}'
 
         if href:
             href = self.checkRefs(href)
@@ -998,11 +994,11 @@ class Textile:
 
         out = []
         if href:
-            out.append('<a href="%s" class="img">' % href)
+            out.append(f'<a href="{href}" class="img">')
         if self.html_type == 'html':
-            out.append('<img src="%s"%s>' % (url, atts))
+            out.append(f'<img src="{url}"{atts}>')
         else:
-            out.append('<img src="%s"%s />' % (url, atts))
+            out.append(f'<img src="{url}"{atts} />')
         if href:
             out.append('</a>')
 
@@ -1021,7 +1017,7 @@ class Textile:
         # text needs to be escaped
         if not self.restricted:
             text = self.encode_html(text)
-        return ''.join([before, self.shelve('<code>%s</code>' % text), after])
+        return ''.join([before, self.shelve(f'<code>{text}</code>'), after])
 
     def fPre(self, match):
         before, text, after = match.groups()
@@ -1035,13 +1031,13 @@ class Textile:
     def doSpecial(self, text, start, end, method=None):
         if method is None:
             method = self.fSpecial
-        pattern = re.compile(r'(^|\s|[\[({>])%s(.*?)%s(\s|$|[\])}])?' % (re.escape(start), re.escape(end)), re.M|re.S)
+        pattern = re.compile(fr'(^|\s|[\[({{>]){re.escape(start)}(.*?){re.escape(end)}(\s|$|[\])}}])?', re.M|re.S)
         return pattern.sub(method, text)
 
     def fSpecial(self, match):
-        """
+        '''
         special blocks like notextile or code
-        """
+        '''
         before, text, after = match.groups()
         if after is None:
             after = ''
@@ -1059,17 +1055,17 @@ class Textile:
 
 
 def textile(text, head_offset=0, html_type='xhtml', encoding=None, output=None):
-    """
+    '''
     this function takes additional parameters:
     head_offset - offset to apply to heading levels (default: 0)
     html_type - 'xhtml' or 'html' style tags (default: 'xhtml')
-    """
+    '''
     return Textile().textile(text, head_offset=head_offset,
                              html_type=html_type)
 
 
 def textile_restricted(text, lite=True, noimage=True, html_type='xhtml'):
-    """
+    '''
     Restricted version of Textile designed for weblog comments and other
     untrusted input.
 
@@ -1084,7 +1080,7 @@ def textile_restricted(text, lite=True, noimage=True, html_type='xhtml'):
     When noimage=True is set (the default):
     Image tags are disabled.
 
-    """
+    '''
     return Textile(restricted=True, lite=lite,
                    noimage=noimage).textile(text, rel='nofollow',
                                             html_type=html_type)
