@@ -1,26 +1,48 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
+from contextlib import suppress
+from functools import partial
+
 from qt.core import (
-    QLineEdit, QAbstractListModel, Qt, pyqtSignal, QObject, QKeySequence, QAbstractItemView,
-    QApplication, QListView, QPoint, QModelIndex, QEvent,
-    QStyleOptionComboBox, QStyle, QComboBox, QTimer, sip)
+    QAbstractItemView,
+    QAbstractListModel,
+    QApplication,
+    QComboBox,
+    QEvent,
+    QKeySequence,
+    QLineEdit,
+    QListView,
+    QModelIndex,
+    QObject,
+    QPoint,
+    QStyle,
+    QStyleOptionComboBox,
+    Qt,
+    QTimer,
+    pyqtProperty,
+    pyqtSignal,
+    sip,
+)
 
 from calibre.constants import ismacos
-from calibre.utils.icu import sort_key, primary_startswith, primary_contains
 from calibre.gui2.widgets import EnComboBox, LineEditECM
 from calibre.utils.config import tweaks
+from calibre.utils.icu import primary_contains, primary_startswith, sort_key
 
 
 def containsq(x, prefix):
     return primary_contains(prefix, x)
 
 
+def hierarchy_startswith(x, prefix, sep='.'):
+    return primary_startswith(x, prefix) or primary_contains(sep + prefix, x)
+
+
 class CompleteModel(QAbstractListModel):  # {{{
 
     def __init__(self, parent=None, sort_func=sort_key, strip_completion_entries=True):
@@ -41,7 +63,7 @@ class CompleteModel(QAbstractListModel):  # {{{
         self.current_prefix = ''
         self.endResetModel()
 
-    def set_completion_prefix(self, prefix):
+    def set_completion_prefix(self, prefix, hierarchy_separator: str = ''):
         old_prefix = self.current_prefix
         self.current_prefix = prefix
         if prefix == old_prefix:
@@ -54,6 +76,11 @@ class CompleteModel(QAbstractListModel):  # {{{
         subset = prefix.startswith(old_prefix)
         universe = self.current_items if subset else self.all_items
         func = primary_startswith if tweaks['completion_mode'] == 'prefix' else containsq
+        if func is primary_startswith and hierarchy_separator:
+            if hierarchy_separator != '.':
+                func = partial(hierarchy_startswith, sep=hierarchy_separator)
+            else:
+                func = hierarchy_startswith
         self.beginResetModel()
         self.current_items = tuple(x for x in universe if func(x, prefix))
         self.endResetModel()
@@ -63,10 +90,19 @@ class CompleteModel(QAbstractListModel):  # {{{
 
     def data(self, index, role):
         if role == Qt.ItemDataRole.DisplayRole:
-            try:
+            with suppress(IndexError):
+                ans = self.current_items[index.row()].replace('\n', ' ')
+                if not self.strip_completion_entries:
+                    ls = ans.lstrip()
+                    if len(ls) < len(ans):
+                        ans = '␣'*(len(ans) - len(ls)) + ls
+                    rs = ans.rstrip()
+                    if len(rs) < len(ans):
+                        ans = rs + '␣'*(len(ans) - len(rs))
+                return ans
+        if role == Qt.ItemDataRole.UserRole:
+            with suppress(IndexError):
                 return self.current_items[index.row()]
-            except IndexError:
-                pass
 
     def index_for_prefix(self, prefix):
         for i, item in enumerate(self.current_items):
@@ -78,6 +114,7 @@ class CompleteModel(QAbstractListModel):  # {{{
 class Completer(QListView):  # {{{
 
     item_selected = pyqtSignal(object)
+    apply_current_text = pyqtSignal()
     relayout_needed = pyqtSignal()
 
     def __init__(self, completer_widget, max_visible_items=7, sort_func=sort_key, strip_completion_entries=True):
@@ -97,6 +134,8 @@ class Completer(QListView):  # {{{
         self.pressed.connect(self.item_chosen)
         self.installEventFilter(self)
         self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
+        self.tab_accepts_uncompleted_text = (tweaks['tab_accepts_uncompleted_text'] and
+                                             not tweaks['preselect_first_completion'])
 
     def hide(self):
         self.setCurrentIndex(QModelIndex())
@@ -106,7 +145,7 @@ class Completer(QListView):  # {{{
         if not self.isVisible():
             return
         self.hide()
-        text = self.model().data(index, Qt.ItemDataRole.DisplayRole)
+        text = self.model().data(index, Qt.ItemDataRole.UserRole)
         self.item_selected.emit(str(text))
 
     def set_items(self, items):
@@ -114,8 +153,8 @@ class Completer(QListView):  # {{{
         if self.isVisible():
             self.relayout_needed.emit()
 
-    def set_completion_prefix(self, prefix):
-        self.model().set_completion_prefix(prefix)
+    def set_completion_prefix(self, prefix, hierarchy_separator: str = ''):
+        self.model().set_completion_prefix(prefix, hierarchy_separator=hierarchy_separator)
         if self.isVisible():
             self.relayout_needed.emit()
 
@@ -143,7 +182,7 @@ class Completer(QListView):  # {{{
         widget = self.parent()
         if widget is None:
             return
-        screen = QApplication.desktop().availableGeometry(widget)
+        screen = widget.screen().availableGeometry()
         h = (p.sizeHintForRow(0) * min(self.max_visible_items, m.rowCount()) + 3) + 3
         hsb = p.horizontalScrollBar()
         if hsb and hsb.isVisible():
@@ -228,6 +267,9 @@ class Completer(QListView):  # {{{
                 if idx.isValid():
                     self.item_chosen(idx)
                     self.hide()
+                elif self.tab_accepts_uncompleted_text:
+                    self.hide()
+                    self.apply_current_text.emit()
                 elif self.model().rowCount() > 0:
                     self.next_match()
                 e.accept()
@@ -295,6 +337,7 @@ class LineEdit(QLineEdit, LineEditECM):
         self.sep = ','
         self.space_before_sep = False
         self.add_separator = True
+        self.hierarchy_separator = ''
         self.original_cursor_pos = None
         completer_widget = (self if completer_widget is None else
                 completer_widget)
@@ -302,18 +345,26 @@ class LineEdit(QLineEdit, LineEditECM):
         self.mcompleter = Completer(completer_widget, sort_func=sort_func, strip_completion_entries=strip_completion_entries)
         self.mcompleter.item_selected.connect(self.completion_selected,
                 type=Qt.ConnectionType.QueuedConnection)
+        self.mcompleter.apply_current_text.connect(self.apply_current_text,
+                type=Qt.ConnectionType.QueuedConnection)
         self.mcompleter.relayout_needed.connect(self.relayout)
         self.mcompleter.setFocusProxy(completer_widget)
         self.textEdited.connect(self.text_edited)
         self.no_popup = False
 
     # Interface {{{
+    def set_sort_func(self, sort_func):
+        self.mcompleter.model().sort_func = sort_func
+
     def update_items_cache(self, complete_items):
         self.all_items = complete_items
 
     def set_separator(self, sep):
         self.sep = sep
 
+    def set_hierarchy_separator(self, sep: str = '') -> None:
+        self.hierarchy_separator = sep
+
     def set_space_before_sep(self, space_before):
         self.space_before_sep = space_before
 
@@ -355,7 +406,7 @@ class LineEdit(QLineEdit, LineEditECM):
         orig = None
         if show_all:
             orig = self.mcompleter.model().current_prefix
-            self.mcompleter.set_completion_prefix('')
+            self.mcompleter.set_completion_prefix('', self.hierarchy_separator)
         if not self.mcompleter.model().current_items:
             self.mcompleter.hide()
             return
@@ -384,7 +435,7 @@ class LineEdit(QLineEdit, LineEditECM):
         complete_prefix = prefix.lstrip()
         if self.sep:
             complete_prefix = prefix.split(self.sep)[-1].lstrip()
-        self.mcompleter.set_completion_prefix(complete_prefix)
+        self.mcompleter.set_completion_prefix(complete_prefix, self.hierarchy_separator)
 
     def get_completed_text(self, text):
         'Get completed text in before and after parts'
@@ -421,6 +472,14 @@ class LineEdit(QLineEdit, LineEditECM):
         self.setCursorPosition(len(before_text))
         self.item_selected.emit(text)
 
+    def apply_current_text(self):
+        if self.sep is not None:
+            txt = str(self.text())
+            sep_pos = txt.rfind(self.sep)
+            if sep_pos:
+                ntxt = txt[sep_pos+1:].strip()
+                self.completion_selected(ntxt)
+
 
 class EditWithComplete(EnComboBox):
 
@@ -437,6 +496,10 @@ class EditWithComplete(EnComboBox):
         self.installEventFilter(self)
 
     # Interface {{{
+
+    def set_sort_func(self, sort_func):
+        self.lineEdit().set_sort_func(sort_func)
+
     def showPopup(self):
         orig = self.disable_popup
         self.disable_popup = False
@@ -445,12 +508,22 @@ class EditWithComplete(EnComboBox):
         finally:
             self.disable_popup = orig
 
+    def keyPressEvent(self, e):
+        if e.key() in (Qt.Key.Key_Down, Qt.Key.Key_Up) and not self.lineEdit().text() and not self.disable_popup:
+            e.accept()
+            self.showPopup()
+            return
+        super().keyPressEvent(e)
+
     def update_items_cache(self, complete_items):
         self.lineEdit().update_items_cache(complete_items)
 
     def set_separator(self, sep):
         self.lineEdit().set_separator(sep)
 
+    def set_hierarchy_separator(self, sep):
+        self.lineEdit().set_hierarchy_separator(sep)
+
     def set_space_before_sep(self, space_before):
         self.lineEdit().set_space_before_sep(space_before)
 
@@ -486,7 +559,16 @@ class EditWithComplete(EnComboBox):
     # }}}
 
     def text(self):
-        return str(self.lineEdit().text())
+        return self.lineEdit().text()
+
+    def set_current_text(self, text):
+        self.setText(text)
+        self.selectAll()
+
+    # Create a Qt user property for the current text so that when this widget
+    # is used as an edit widget in a table view it selects all text, as
+    # matching the behavior of all other Qt widgets.
+    current_text = pyqtProperty(str, fget=text, fset=set_current_text, user=True)
 
     def selectAll(self):
         self.lineEdit().selectAll()
@@ -525,14 +607,15 @@ class EditWithComplete(EnComboBox):
 
 if __name__ == '__main__':
     from qt.core import QDialog, QVBoxLayout
+
     from calibre.gui2 import Application
     app = Application([])
     d = QDialog()
     d.setLayout(QVBoxLayout())
     le = EditWithComplete(d)
     d.layout().addWidget(le)
-    items = ['one', 'otwo', 'othree', 'ooone', 'ootwo', 'other', 'odd', 'over', 'orc', 'oven', 'owe',
+    items = ['oane\n line2\n line3', 'otwo', 'othree', 'ooone', 'ootwo', 'other', 'odd', 'over', 'orc', 'oven', 'owe',
         'oothree', 'a1', 'a2','Edgas', 'Èdgar', 'Édgaq', 'Edgar', 'Édgar']
     le.update_items_cache(items)
     le.show_initial_value('')
-    d.exec_()
+    d.exec()
