@@ -350,18 +350,18 @@ def atomic_write(base, name, content):
 
 
 def run_rapydscript_tests():
+    from calibre.gui2 import must_use_qt
+    must_use_qt()
+    from calibre.utils.webengine import create_script, insert_scripts, secure_webengine, setup_default_profile, setup_fake_protocol, setup_profile
+    setup_fake_protocol()
+    setup_default_profile()
     from urllib.parse import parse_qs
 
     from qt.core import QApplication, QByteArray, QEventLoop, QUrl
     from qt.webengine import QWebEnginePage, QWebEngineProfile, QWebEngineScript, QWebEngineUrlRequestJob, QWebEngineUrlSchemeHandler
 
     from calibre.constants import FAKE_HOST, FAKE_PROTOCOL
-    from calibre.gui2 import must_use_qt
     from calibre.gui2.viewer.web_view import send_reply
-    from calibre.utils.webengine import create_script, insert_scripts, secure_webengine, setup_default_profile, setup_fake_protocol, setup_profile
-    must_use_qt()
-    setup_fake_protocol()
-    setup_default_profile()
 
     base = base_dir()
     rapydscript_dir = os.path.join(base, 'src', 'pyj')
@@ -430,6 +430,8 @@ def run_rapydscript_tests():
 
     tester = Tester()
     result = tester.spin_loop()
+    if result is None:
+        result = 1
     raise SystemExit(int(result))
 
 
